<?php

namespace src\fortnox\api;

if ( !defined( 'ABSPATH' ) ) die();

use Exception;
use src\fortnox\WF_Plugin;

class WF_Auth
{
    /**
     * Get access token
     * @return mixed
     * @throws Exception
     */
	public static function get_access_token()
	{
		$auth_code = get_option( 'fortnox_auth_code'  );
		$client_secret = WF_Plugin::CLIENT_SECRET;

		if( empty( $auth_code ) || empty( $client_secret ) ){
            throw new Exception( "Authorisation code is empty." );
        }

        $response = wp_remote_get( "https://api.fortnox.se/3/", array( 'headers' => [
            'Authorization-Code' => $auth_code,
            'Client-Secret' => $client_secret,
            'Content-Type' => 'application/json',
            'Accept' => 'application/json'
        ] ) );

        fortnox_write_log( "get_access_token: " );
        fortnox_write_log( $response );

        $response = json_decode( $response['body'] );

		if( ! empty( $response->ErrorInformation ) ){
            throw new Exception( "{$response->ErrorInformation->Message} (Felkod: {$response->ErrorInformation->Code})" );

        }

		if( ! empty( $response->Authorization->AccessToken ) ){
            update_option( 'fortnox_access_token', $response->Authorization->AccessToken );
            update_option( 'fortnox_connected_organization_number', WF_Company_Information::get_organization_number() );
        }

		return $response->Authorization->AccessToken;
	}
}

