<?php

namespace src\fortnox\api;

if ( !defined( 'ABSPATH' ) ) die();

use Exception;
use src\fortnox\WF_Plugin;
use WC_Order;

class WF_Full_Refund_Invoices  extends WF_Refund_Invoices{

    /**
     * @param $refunded_order
     * @param $invoice_number
     * @throws Exception
     */
    public static function post_process( $order, $refunded_invoice_number ){
        if ( get_option( 'fortnox_auto_post_refund_invoice'  ) ) {

            if ( get_option( 'fortnox_has_warehouse_module'  ) ) {
                WF_Invoices::mark_as_ready( $refunded_invoice_number );
            }
            WF_Request::put("/invoices/" . $refunded_invoice_number . "/bookkeep" );
            WF_Refund_Invoices::set_credit_invoice_reference( $order, $refunded_invoice_number );

            if (get_option( 'fortnox_auto_set_refund_invoice_as_paid' ) ) {

                if ( floatval( $order->get_total() ) != 0.0) {
                    self::make_refund_invoice_payment( $order, $refunded_invoice_number );
                }
            }
        }
    }

    /** Controller function for handling a refund
     * @param \WC_Order $order
     * @param int $refunded_order_id
     * @return mixed
     * @throws \Exception
     */
    public static function create_full_credit_invoice( $order ){

        $credit_invoice_number = WF_Refund_Invoices::create_credit_invoice( $order );
        if( ! $credit_invoice_number ){
            return;
        }
        try{
            WF_Refunds::set_order_refund_as_synced( $order->get_id() );
            foreach (  $order->get_refunds() as $refund ){
                WF_Refunds::set_order_refund_as_synced( $refund->get_id() );
            }

            self::set_refund_invoice_number( $order, $credit_invoice_number );
            $order->add_order_note( __( 'Refund sent to Fortnox, Invoice number: ', WF_Plugin::TEXTDOMAIN ) . $credit_invoice_number );

        } catch( \Exception $error ) {
            WF_Orders::add_order_log( $order, $error );
            WF_Orders::set_order_notice_flag( $order );
            throw new \Exception( $error->getMessage(), $error->getCode() );
        }

        do_action( 'wf_order_after_full_refund', $order );

        return $credit_invoice_number;
    }

    /**
     * Sets invoiced as paid
     * @param WC_Order $order
     * @return void
     * @throws \Exception
     */
    public static function make_refund_invoice_payment( $order, $invoice_number ){

        if( ! is_a( $order, 'WC_Order_Refund' ) ) {
            $order->calculate_totals();
            $payment_method = $order->get_payment_method();
        }
        else{
            $parent_order = wc_get_order( $order->get_parent_id() );
            $payment_method = $parent_order->get_payment_method();
        }

        $invoice_payment = apply_filters( 'wf_invoice_payment_before_create_or_update', [
            "AmountCurrency"            => (-1) * $order->get_total(),
            "InvoiceNumber"             => $invoice_number,
            "PaymentDate"               => substr( $order->get_date_created(), 0, 10 ),
            "ModeOfPaymentAccount"      => WF_Invoices::get_payment_account( $payment_method )
        ], $order );

        $response = WF_Request::post( "/invoicepayments", [
            "InvoicePayment" => $invoice_payment
        ] );

        WF_Request::put("/invoicepayments/" . $response->InvoicePayment->Number . "/bookkeep" );
    }
}