<?php

namespace src\fortnox;

if ( !defined( 'ABSPATH' ) ) die();

class WF_Utils
{

    /**
     * Returns tax rates
     * @param \WC_Product $product
     * @param $country_code
     * @return int
     */
    public static function get_wc_tax_rate( $product, $country_code ){

        if ( empty( $country_code ) ){
            $country_code = 'SE';
        }

        foreach( \WC_Tax::get_rates_for_tax_class( $product->get_tax_class() ) as $tax_rate ){
            if ( $country_code == $tax_rate->tax_rate_country ){
                return $tax_rate->tax_rate;
            }
        }

        foreach( \WC_Tax::get_rates_for_tax_class( $product->get_tax_class() ) as $tax_rate ){

            if( '' == $tax_rate->tax_rate_country ){
                return $tax_rate->tax_rate;
            }
        }
        return 0;
    }

    /** Filter a shipping zone, returns zone_id if country in is in zone
     * @param $zone
     * @param $shipping_country
     * @return mixed
     */
    private static function filter_zone( $zone, $shipping_country ){
        return array_reduce( $zone['zone_locations'], function( $zone_id, $zone ) use ( $shipping_country ) {

            if( $shipping_country == $zone->code ){
                return $zone['zone_id'];
            }
            return $zone_id;
        });
    }

    /** Filter a shipping zone, returns zone_id if country in is in zone
     * @param $zone
     * @return array
     */
    private static function get_zone_codes( $zone_locations ){
        return array_map( function( $zone_location ) {
            return $zone_location->code ;
        }, $zone_locations );
    }

    /** Filter a shipping zone, returns zone_id if country in is in zone
     * @param $shipping_country
     * @return mixed
     */
    public static function get_zone_id( $shipping_country ){
        foreach ( \WC_Shipping_Zones::get_zones() as $zone ){
            if ( in_array( $shipping_country, self::get_zone_codes( $zone['zone_locations'] ) ) )  {
                return $zone['zone_id'];
            }
        }

    }

    /**
     * @param int $order_id
     * @return array | bool
     */
    public static function get_vat_number( $order_id ){

        $vat_number = get_post_meta( $order_id, '_vat_number', true );

        // Set customer VAT type based on country
        if ( ! empty( $vat_number ) ) {
            return $vat_number;
        }

        $vat_number = get_post_meta( $order_id, '_billing_vat_number', true );
        // Set customer VAT type based on country
        if ( ! empty( $vat_number ) ) {
            return $vat_number;
        }

        $vat_number = get_post_meta( $order_id, apply_filters( 'wf_eu_vat_meta_key', '__'  ), true );

        // Set customer VAT type based on country
        if ( ! empty( $vat_number ) ) {
            return $vat_number;
        }

        return apply_filters( 'wf_eu_vat_number', false );
    }

    public static function vat_number_is_valid( $order ){
        return apply_filters( 'wf_eu_vat_number_is_valid', wc_string_to_bool( get_post_meta( $order->get_id(), '_vat_number_is_valid', true ) ), $order );
    }

    /**
     * @param $order_id
     * @return \stdClass|\WC_Order[]
     */
    public static function get_refunds( $order_id ){
        return wc_get_orders(
            array(
                'type'   => 'shop_order_refund',
                'parent' => intval( $order_id ),
                'return' => 'ids',
                'limit'  => -1,
            )
        );
    }

    public static function maybe_mail_error( $order_id, $error_message )
    {

        if ( get_option( 'fortnox_email_synchronization_errors' ) ){

            $message = __("Something went wrong when synchronizing order ID: {$order_id} to Fortnox.\n Error message: \n", WF_Plugin::TEXTDOMAIN);
            $message .= $error_message;
            $to = get_option('admin_email');
            $subject = __("Fortnox error", WF_Plugin::TEXTDOMAIN);
            $headers = 'From: ' . $to . "\r\n" .
                "Reply-To: <>\r\n";

            return wp_mail($to, $subject, strip_tags($message), $headers);
        }
    }
}
